

/*	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	-----------------------------------------------------------------
	|	TUTORIAL 5:													|
	|		MDE4CPP Plugin Framework for Ecore					|
	-----------------------------------------------------------------
	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*/

//Incude standard libraries
#include <iostream>

//Include header files for plugin framework
#include "pluginFramework/PluginFramework.hpp"
#include "pluginFramework/MDE4CPPPlugin.hpp"
#include "pluginFramework/EcoreModelPlugin.hpp"

//Include header files for ecore
#include "ecore/EClass.hpp"
#include "ecore/EObject.hpp"
#include "ecore/EFactory.hpp"
#include "ecore/EPackage.hpp"
#include "abstractDataTypes/SubsetUnion.hpp"

using namespace ecore;

int main()
{
	//Create an instance of the plugin framework
	std::shared_ptr<PluginFramework> pluginFramework = PluginFramework::eInstance();

	//Search for our model plugin and create an MDE4CPPPlugin object for it
	std::shared_ptr<MDE4CPPPlugin> plugin = pluginFramework->findPluginByName("uniModelExample_ecore");
	
	//Check for nullptr
	if (plugin){
		std::cout << "- MDE4CPPPlugin '" << plugin->eNAME() << "' found" << std::endl;
		
		//Cast MDE4CPPPlugin to EcoreModelPlugin (to be able to access the models Package & Factory)
		std::shared_ptr<EcoreModelPlugin> ecorePlugin = std::dynamic_pointer_cast<EcoreModelPlugin>(plugin);
		
		//Check for nullptr
		if(ecorePlugin){
			std::cout<<"- EcoreModelPlugin created from MDE4CPPPlugin"<<std::endl<<std::endl;
			
			//Get the models package
			std::shared_ptr<EPackage> uniModelPackage = ecorePlugin->getEPackage();
			//Get the models factory
			std::shared_ptr<EFactory> uniModelFactory = ecorePlugin->getEFactory();
			

	//-----------------------------------------------------------
	//		Using eSet to set values of attributes and references
	//-----------------------------------------------------------
			//Create University and Faculty objects 
			std::shared_ptr<EObject> uniObj = ecorePlugin->create("University");
			std::shared_ptr<EObject> facObj = ecorePlugin->create("Faculty");
			
			//Set EAttribute for multiplicity [0..1]
			std::string facName = "Computer Science";
			//Use eSet to set the value for attribute "name" of faculty (value passed as "Any" type)
			facObj->eSet(facObj->eClass()->getEStructuralFeature("name"), eAny(facName));
			
			//Set EReference for multiplicity [0..1]
			facObj->eSet(facObj->eClass()->getEStructuralFeature("university"), eAny(uniObj));
			
			//Set EReference for multiplicity [0..*]
			//Use eGet to create container of all faculties of our university (returned as an "Any" type)
			Any anyAllFaculties = uniObj->eGet(uniObj->eClass()->getEStructuralFeature("faculties"));
			//Casting the "Any" type to container of EObjects
			std::shared_ptr<Bag<EObject> > allFaculties = anyAllFaculties->get<std::shared_ptr<Bag<EObject> > >();
			//Add our new faculty object to container
			allFaculties->add(facObj);
			//Set new container for universitys "faculties" (value passed as an "Any" type)
			uniObj->eSet(uniObj->eClass()->getEStructuralFeature("faculties"), eAny(allFaculties));
			
			//Create new model class instance and set some attributes/references
			std::shared_ptr<EObject> studObj = ecorePlugin->create("Student");
			//Set students name attribute
			std::string studName = "John Doe";
			studObj->eSet(studObj->eClass()->getEStructuralFeature("name"), eAny(studName));
			//Set students matriculation number attribute
			int studMatNum = 98765;
			studObj->eSet(studObj->eClass()->getEStructuralFeature("matNum"), eAny(studMatNum));
			
			//Add the student to all students of our faculty
			//Get container of all students as “Any” type and cast it
			Any anyAllStudents = facObj->eGet(facObj->eClass()->getEStructuralFeature("students"));
			std::shared_ptr<Bag<EObject> > allStudents = anyAllStudents->get<std::shared_ptr<Bag<EObject> > >();
			//Add the student to container and pass it back using eSet()
			allStudents->add(studObj);
			facObj->eSet(facObj->eClass()->getEStructuralFeature("students"), eAny(allStudents));
			
		
	//---------------------------------------------------------------------------
	//		Using eGet to get values of attributes and references and output them
	//---------------------------------------------------------------------------
			//Get all faculties of our model
			anyAllFaculties = uniObj->eGet(uniObj->eClass()->getEStructuralFeature("faculties"));
			allFaculties = anyAllFaculties->get<std::shared_ptr<Bag<EObject> > >();
			std::cout<<"Your University:" <<std::endl<<std::endl;
			
			//Loop over all faculties
			for(Bag<EObject>::const_iterator facIt = allFaculties->begin(); facIt != allFaculties->end(); facIt++){
				//Use eGet to get the value of attribute "name" of the current faculty object (returned as an "Any" type)
				Any anyFacultyName = (*facIt)->eGet((*facIt)->eClass()->getEStructuralFeature("name"));
				//Casting the "Any" type to std::string
				std::cout <<"Faculty "<<anyFacultyName->get<std::string>()<<std::endl;
				
				//Get all students of the current faculty
				anyAllStudents = (*facIt)->eGet((*facIt)->eClass()->getEStructuralFeature("students"));
				allStudents = anyAllStudents->get<std::shared_ptr<Bag<EObject> > >();
				
				//Loop over all students
				for(Bag<EObject>::const_iterator studIt = allStudents->begin(); studIt != allStudents->end(); studIt++){
					Any anyStudentName = (*studIt)->eGet((*studIt)->eClass()->getEStructuralFeature("name"));
					Any anyStudentMatNum = (*studIt)->eGet((*studIt)->eClass()->getEStructuralFeature("matNum"));
					//Casting the "Any" types to std::string and int
					std::cout<<"\t-Student "<<anyStudentName->get<std::string>()<<", "<<anyStudentMatNum->get<int>()<<std::endl;
				}
				
			}

		}

	}
	
    return 0;
}



